<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/generation/Generator.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
class Generators {
   private $gDefault;
   
   private $gGenerators;
   
   private $gDOM;
      
   
   
   
   public function __construct()
   {
   }
   
   
   
   public function parse( DOMNode $inNode )
   {
      
      
      $this->gDOM = new DOMDocument;
      $this->gDOM->appendChild( $this->gDOM->importNode( $inNode, 1 ) );
   
      
      $xPath = new domxpath( $this->gDOM );
      $xPath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $component = $xPath->query( "//m:generator" );
      
      foreach ( $component as $node ) { 
         $name = $node->getAttribute( "name" );
         $src = $node->getAttribute( "src" );
         $isCachable = Utilities::normalizeBoolean( $node->getAttribute( "cachable" ) );
         if ( !strlen( $name ) or !strlen( $src ) ) {
            throw new UserException(
               "Component generator must have name and src attributes defined",
               PalooseException::SITEMAP_GENERATOR_PARSE_ERROR,
               $this->gDOM );
         }
         
         
         if ( strstr( $src, Environment::getPathSeparator() ) == 1 ) {
            $packageFilename = $src;
         } else if ( !strstr( $src, ":" . Environment::getPathSeparator() ) ) {
            
            $packageFilename = PALOOSE_LIB_DIRECTORY . Environment::getPathSeparator() . $src;
         } else {
            
            $packageFilename = StringResolver::expandPseudoProtocols( $src );
         }
         $package = Utilities::stripFileName( $packageFilename );
         
         $path = Utilities::stripPath( StringResolver::expandPseudoProtocols( $packageFilename ) );
         
         
         if ( $this->gGenerators != NULL and array_key_exists( $name, $this->gGenerators ) ) {
         } else {
         }
         require_once( $path . $package . ".php" );
         
         $generatorName = "_" . $package;
         $generator = new $generatorName( $name, $path.$package, $node, $isCachable );
         
         try {
            $generator->isValid();
            $this->gGenerators[ $name ] = $generator;
         } catch( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $this->gDOM );
         }
      }
   }
   
   
   
   public function setDefault( $inDefault )
   {
      if ( !( array_key_exists( $inDefault, $this->gGenerators ) ) ) {
           throw new UserException(
              "Default generator '". $inDefault . "' not found",
              PalooseException::SITEMAP_GENERATOR_PARSE_ERROR,
              $this->gDOM );
      }
      $this->gDefault = $inDefault;
   }
   
   
   
   public function getDefault()
   {
      return $this->gDefault;
   }
   
   
   
   public function getGenerator( $inName )
   {
      if ( !( array_key_exists( $inName, $this->gGenerators ) ) ) {
           throw new UserException(
              "Generator '". $inName . "' not found",
              PalooseException::SITEMAP_GENERATOR_PARSE_ERROR,
              $this->gDOM );
      } else {
         return $this->gGenerators[ $inName ];
      }
   }
   
   
   
   public function getGeneratorClassName( $inType )
   {
      if ( !( array_key_exists( $inType, $this->gGenerators ) ) ) {
           throw new UserException(
              "Generator '". $inType . "' not found",
              PalooseException::SITEMAP_GENERATOR_PARSE_ERROR,
             $this->gDOM );
      } else {
         $className = $this->gGenerators[ $inType ]->getPackageName();
         return $className;
      }
   }
   
   
   
   public function toString()
   {
      $mess = "  <generators default='{$this->gDefault}'>\n";
      foreach ( $this->gGenerators as $node ) {
         $mess .= $node->toString();
      }
      $mess .= "  </generators>\n";
      return $mess;
   }
}
?>
