<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
#[AllowDynamicProperties]
class Call extends PipeElement implements PipeElementInterface
{
   
   private $gResourceName;
   
   
   private $gCallType;
   const RESOURCE = 1;
   const FUNC = 2;
    
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      $this->gGenericType = PipeElement::CALL;
      $this->gSitemapSrc = $inSrc;
      $this->gType = $inType;
      $this->gComponent = $inComponent;
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      $this->gResourceName = Utilities::getXPathListStringItem( 0, $xpath, "//@resource" );
      if ( strlen( $this->gResourceName ) > 0 ) {
         $this->gCallType = self::RESOURCE;
           return;
      }
      $this->gResourceName = Utilities::getXPathListStringItem( 0, $xpath, "//@function" );
      if ( strlen( $this->gResourceName ) > 0 ) {
         $this->gCallType = self::FUNC;
           return;
      }
        throw new UserException( "Call must have a name/function attribute defined", PalooseException::SITEMAP_CALL_PARSE_ERROR, $inDOM );
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      try {
         switch ( $this->gCallType ) {
         
            case self::RESOURCE :
               
               $sitemap = Environment::$sitemapStack->peek();
               $resources = $sitemap->getResources();
               $resource = $resources->getResource( $this->gResourceName );
               $pipe = $resource->getPipe();
               return $pipe->run( $inVariableStack, $inURL, $inQueryString, $inDOM );
            case self::FUNC :
               
               $pair = explode( "::", $this->gResourceName );
               if ( $pair != NULL and count( $pair ) == 2 ) {
                  
                  $class = $pair[0];
                  $method = $pair[1];
               
                  if ( !empty( $this->gParameters->getParameterList() ) ) {
                     $parameterList = $this->gParameters->getParameterList();
                     foreach ( $parameterList as $name => $value ) {
                        $expandedValue = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $value ) );
                        
                        $paramsModule = Environment::$modules[ 'params' ];
                        $paramsModule->add( $name, $value );
                     }
                  }
                  
                  
                  $instance = new $class();
                  
                  $instance->$method();
               } else {
                  throw new RunException( "Must have class and method defined in function call", NULL );
               }
         }
      
      } catch ( ExitException $e ) {
           throw new ExitException();
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
   public function getResourceName()
   {
      return $this->gResourceName;
   }
   
   
   
   
   public function getCallType()
   {
      return $this->gCallType;
   }
   
   
   
   
   public function toString()
   {
      switch ( $this->gCallType ) {
         case self::RESOURCE : return "<call name='{$this->gResourceName}' />\n";
         case self::FUNC : return "<call function='{$this->gResourceName}' />\n";
      }
   }
}
?>
