<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/caching/CacheFile.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/../plugins/scssphp/scss.inc.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/SCSSException.php" );
use ScssPhp\ScssPhp\Compiler;
use ScssPhp\ScssPhp\OutputStyle;
use ScssPhp\ScssPhp\Version;
define( "PALOOSE_SCSS_SRC_ATTRIB", "src" );
define( "PALOOSE_SCSS_DST_ATTRIB", "dst" );
define( "PALOOSE_SCSS_CACHE", "scss_cache" );
define( "PALOOSE_SCSS_COMPACT_ATTRIB", "compact" );
class SCSSCompiler extends TransformerPipeElement implements PipeElementInterface
{
    
    
    
    
    private $gCSSFolder;
    
    
    private $gSCSSFolder;
    
    
    private $gCacheFolder;
    
    
    private $gIsCompact;
    
    
    public $gSCSS;
    
    
    
    
    
    public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
    {
        $gSCSS = new Compiler();
        try {
            parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
            
            
            $this->gCSSFolder = $inComponent->gDefaultCSSFolder;
            $this->gSCSSFolder = $inComponent->gDefaultSCSSFolder;
            $this->gCacheFolder = $inComponent->gCacheFolder;
            $this->gIsCompact = $inComponent->gIsCompact;
            
            
            $this->gSCSS = $inComponent->gSCSS;
            
        } catch ( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
        
    }
    
    
    
    
    
    public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
    {
        parent::run( $inVariableStack, $inURL, $inQueryString, $inDOM );
        
        
        
        $parameters = [];
        if ( !empty( $this->gParameters->getParameterList() ) ) {
            $parameterList = $this->gParameters->getParameterList();
            foreach ( $parameterList as $name => $value ) {
                $expandedValue = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $value ) );
                $parameters[ $name ] =  $value;
            }
        }
        
        if ( isset( $parameters[ PALOOSE_SCSS_SRC_ATTRIB ] ) && strlen( $parameters[ PALOOSE_SCSS_SRC_ATTRIB ] ) > 0 ) {
            $this->gSCSSFolder = StringResolver::expandPseudoProtocols( $parameters[ PALOOSE_SCSS_SRC_ATTRIB ] );
        }
        if ( isset( $parameters[ PALOOSE_SCSS_DST_ATTRIB ] ) && strlen( $parameters[ PALOOSE_SCSS_DST_ATTRIB ] ) > 0 ) {
            $this->gCSSFolder = StringResolver::expandPseudoProtocols( $parameters[ PALOOSE_SCSS_DST_ATTRIB ] );
        }
        
        
        if ( !file_exists( $this->gSCSSFolder ) ) {
            return $inDOM;
        }
        
        if ( !file_exists( $this->gCSSFolder ) ) {
            mkdir( $this->gCSSFolder, 0767 );
        }
        
        $this->gCacheFolder = Utilities::join( $this->gSCSSFolder, PALOOSE_SCSS_CACHE );
        if ( !file_exists( $this->gCacheFolder ) ) {
            mkdir( $this->gCacheFolder, 0767 );
        }
        
        $newDocument = $this->serve( $inDOM );
        return $newDocument;
    }
    
    
    
    
    
    public function serve( DOMDocument $inDOM )
    {
        
        $fileList = $this->findFiles();
        
        foreach ( $fileList as $file ) {
            
            $output = $this->cacheName( $file );
            
            $cssFileName = $this->gCSSFolder . Environment::getPathSeparator() . UTILITIES::stripExtension( $file ) . ".css";
            
            if ( $this->getIfNoneMatchHeader() == null ) {
                $etag = $noneMatch = $this->getIfNoneMatchHeader();
            } else {
                $etag = $noneMatch = trim( $this->getIfNoneMatchHeader(), '"' );
            }
            
            if ( $this->needsCompile( $output, $etag ) || !file_exists( $cssFileName ) ) {
                try {
                    list( $css, $etag ) = $this->compile( UTILITIES::join( $this->gSCSSFolder, $file ), $output );
                    $lastModified = gmdate('r', filemtime( $output ) );
                    
                    
                    
                    file_put_contents( $cssFileName, $css ) ;
                } catch ( SCSSException $e ) {
                    
                    
                    $data = $inDOM->saveXML() . "\n";
                    $bodyPattern = "/<body([^>]*)>/i";
                    
                    $errorData = "<body>\n" . $this->createErrorBlock( $e );
                    $inDOM->loadXML( preg_replace( $bodyPattern, $errorData, $data ) );
                }
            }
        }
        return $inDOM;
    }
    
    
    
    
    
    protected function compile( $in, $out )
    {
        $start = microtime( true );
        try {
            $cssResult = $this->gSCSS->compileString( file_get_contents( $in ), $in );
            $returnedCSString = $cssResult->getCss();
            $elapsed = round( ( microtime( true ) - $start ), 4 );
            
            $v    = Version::VERSION;
            $t    = gmdate( 'r' );
            $css  = "/* compiled within Paloose by scssphp $v on $t ({$elapsed}s) */\n\n" . $returnedCSString;
            $etag = md5( "" );
            
            file_put_contents( $out, $css );
            file_put_contents(
                              $this->metadataName( $out ),
                              serialize([
                                        'etag'    => $etag,
                                        'imports' => $cssResult->getIncludedFiles(),
                                        'vars'    => crc32(serialize($this->gSCSS->getVariables())),
                                        ])
                              );
            
            return [ $css, $etag ];
        } catch ( Exception $e ) {
            throw new SCSSException( 'Compile error ' . $e->getMessage(), PalooseException::SCSS_COMPILE_ERROR );
        }
    }
    
    
    
    
    
    protected function createErrorBlock( SCSSException $error )
    {
        return "<div style='background: red; color: white; width: 500px; font-size: 14px; display: block !important; font-family: mono; padding: 1em;'>\n" .
        $error->getMessage(). "</div>";
    }
    
    
    
    
    
    protected function cacheName( $fname )
    {
        return Utilities::join( $this->gCacheFolder, md5( $fname ) . '.css' );
    }
    
    
    
    
    
    protected function metadataName( $out )
    {
        return $out . '.meta';
    }
    
    
    
    
    
    protected function needsCompile( $out, &$etag )
    {
        if ( !is_file( $out ) ) {
            return true;
        }
        
        $mtime = filemtime( $out );
        $metadataName = $this->metadataName( $out );
        if ( is_readable( $metadataName ) ) {
            $metadata = unserialize( file_get_contents( $metadataName ) );
            
            foreach ( $metadata['imports'] as $import => $originalMtime ) {
                if ( file_exists( $import ) ) {
                    $currentMtime = filemtime( $import );
                    if ( $currentMtime !== $originalMtime || $currentMtime > $mtime ) {
                        return true;
                    }
                }
            }
            
            $metaVars = crc32( serialize( $this->gSCSS->getVariables() ) );
            if ( $metaVars !== $metadata['vars'] ) {
                return true;
            }
            $etag = $metadata['etag'];
            return false;
        }
        return true;
    }
    
    
    
    
    
    protected function getIfModifiedSinceHeader()
    {
        $modifiedSince = null;
        if ( isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) ) {
            $modifiedSince = $_SERVER['HTTP_IF_MODIFIED_SINCE'];
            if ( false !== ( $semicolonPos = strpos( $modifiedSince, ';') ) ) {
                $modifiedSince = substr( $modifiedSince, 0, $semicolonPos );
            }
        }
        return $modifiedSince;
    }
    
    
    
    
    
    protected function getIfNoneMatchHeader()
    {
        $noneMatch = null;
        if ( isset( $_SERVER['HTTP_IF_NONE_MATCH'] ) ) {
            $noneMatch = $_SERVER['HTTP_IF_NONE_MATCH'];
        }
        return $noneMatch;
    }
    
    
    
    
    
    public function compileFile($in, $out = null)
    {
        if ( !is_readable( $in ) ) {
            throw new SCSSException( 'load error: failed to find ' . $in, SCSS_COMPILE_ERROR );
        }
        
        $pi = pathinfo( $in );
        $this->scss->addImportPath( $pi['dirname'] . '/' );
        $compiled = $this->gSCSS->compile( file_get_contents( $in ), $in );
        
        if ( $out !== null ) {
            return file_put_contents( $out, $compiled );
        }
        
        return $compiled;
    }
    
    
    
    
    
    public function checkedCompile( $in, $out )
    {
        if ( !is_file( $out ) || filemtime( $in ) > filemtime( $out ) ) {
            $this->compileFile( $in, $out );
            return true;
        }
        return false;
    }
    
    
    
    
    
    protected function inputName()
    {
        switch ( true ) {
            case isset( $_GET['p'] ):
                return $_GET['p'];
                
            case isset( $_SERVER['PATH_INFO'] ):
                return $_SERVER['PATH_INFO'];
                
            case isset( $_SERVER['DOCUMENT_URI'] ):
                return substr( $_SERVER['DOCUMENT_URI'], strlen($_SERVER['SCRIPT_NAME'] ) );
        }
    }
    
    
    
    
    
    protected function findFiles()
    {
        
        $fileList = [];
        $rawFileList = array_diff( scandir( $this->gSCSSFolder ), array('..', '.') );
        
        foreach ( $rawFileList as $file ) {
            if ( substr( $file, 0, 1 ) !== "_"  && substr( $file, -5 ) === '.scss' ) {
                $fullFileName = UTILITIES::join( $this->gSCSSFolder, $file );
                if ( is_file( $fullFileName ) && is_readable( $fullFileName ) ) {
                    array_push( $fileList, $file );
                }
            }
        }
        return $fileList;
    }
    
    
    
    
    
    public function checkedCachedCompile($in, $out, $force = false)
    {
        if ( !is_file($in) || ! is_readable($in)) {
            throw new SCSSException( "Invalid or unreadable input file specified.", PalooseException.SCSS_FILE_NOT_FOUND_ERROR );
        }
        
        if (is_dir($out) || ! is_writable(file_exists($out) ? $out : dirname($out))) {
            throw new SCSSException( "Invalid or unwritable output file specified.", PalooseException.SCSS_FILE_NOT_FOUND_ERROR );
        }
        
        if ($force || $this->needsCompile($out, $etag)) {
            list($css, $etag) = $this->compile($in, $out);
        } else {
            $css = file_get_contents($out);
        }
        
        return $css;
    }
    
    
    
    
    
    public function cachedCompile($in, $force = false)
    {
        
        $root = null;
        
        if (is_string($in)) {
            $root = $in;
        } elseif (is_array($in) and isset($in['root'])) {
            if ($force or ! isset($in['files'])) {
                
                
                
                $root = $in['root'];
            } elseif (isset($in['files']) and is_array($in['files'])) {
                foreach ($in['files'] as $fname => $ftime) {
                    if (! file_exists($fname) or filemtime($fname) > $ftime) {
                        
                        
                        $root = $in['root'];
                        break;
                    }
                }
            }
        } else {
            return null;
        }
        
        if ($root !== null) {
            
            $out = [];
            $out['root'] = $root;
            $out['compiled'] = $this->compileFile($root);
            $out['files'] = $this->gSCSS->getParsedFiles();
            $out['updated'] = time();
            return $out;
        } else {
            
            
            return $in;
        }
    }
    
    
    
    
    
    public function toString()
    {
        return parent::toStringWithType( $this->gGenericType );
    }
    
}
class _SCSSCompiler extends Transformer {
    
    
    
    
    public $gDefaultCSSFolder;
    
    
    public $gDefaultSCSSFolder;
    
    
    public $gSCSSserver;
    
    
    public $gIsCompact;
    
    
    public $gSCSS;
    
    
    public $gOutputStyle;
    
    
    public $gCacheFolder;
    
    
    public $gShowErrorsAsCSS;
    
    
    
    
    
    public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
    {
        parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
        
        $this->gPackageName = "SCSSCompiler";
        
        $this->gDefaultSCSSFolder = "context://resources/scss";
        $this->gDefaultCSSFolder = "context://resources/styles";
        $this->gIsCompact = false;
        
        if ( isset( $this->gParameters[ PALOOSE_SCSS_SRC_ATTRIB ] ) && strlen( $this->gParameters[ PALOOSE_SCSS_SRC_ATTRIB ] ) > 0 ) {
            $this->gDefaultSCSSFolder = StringResolver::expandPseudoProtocols( $this->gParameters[ PALOOSE_SCSS_SRC_ATTRIB ] );
        }
        
        if ( isset( $this->gParameters[ PALOOSE_SCSS_DST_ATTRIB ] ) && strlen( $this->gParameters[ PALOOSE_SCSS_DST_ATTRIB ] ) > 0 ) {
            $this->gDefaultCSSFolder = StringResolver::expandPseudoProtocols( $this->gParameters[ PALOOSE_SCSS_DST_ATTRIB ] );
        }
        
        if ( isset( $this->gParameters[ PALOOSE_SCSS_COMPACT_ATTRIB ] ) && strlen( $this->gParameters[ PALOOSE_SCSS_COMPACT_ATTRIB ] ) > 0 ) {
            $this->gIsCompact = UTILITIES::normalizeBoolean( $this->gParameters[ PALOOSE_SCSS_COMPACT_ATTRIB ] );
        }
        
        $this->gDefaultSCSSFolder = StringResolver::expandPseudoProtocols( $this->gDefaultSCSSFolder );
        $this->gDefaultCSSFolder = StringResolver::expandPseudoProtocols( $this->gDefaultCSSFolder);
        
        if ( !isset( $this->gSCSS ) ) {
            $this->gSCSS = new Compiler();
            $this->gSCSS->setImportPaths( $this->gDefaultSCSSFolder );
        }
        
        if ( $this->gIsCompact ) {
            $this->gSCSS->setOutputStyle( OutputStyle::COMPRESSED );
        } else {
            $this->gSCSS->setOutputStyle( OutputStyle::EXPANDED );
        }
        
        $this->gShowErrorsAsCSS = false;
        date_default_timezone_set( 'UTC' );
        
        
    }
    
}
?>
