<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/AggregatorPart.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/RunException.php" );
#[AllowDynamicProperties]
class Aggregator extends PipeElement implements PipeElementInterface
{
    
   private $gElement;
    
   private $gParts;
    
   
   
   
    
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      
      parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      $this->gGenericType = PipeElement::AGGREGATOR;
      $aggrNodeList = $inDOM->getElementsByTagName( "aggregate" );
      $this->gElement = $aggrNodeList->item( 0 )->getAttribute( "element" );
      if ( strlen ( $this->gElement ) == 0 ) {
         throw new UserException(
            "Must have an associated element attribute with aggregate",
            PalooseException::SITEMAP_AGGREGATE_PARSE_ERROR,
            $inDOM );
      }
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $part = $xpath->query( "//m:part" );
      $idx = 0;
      $this->gPartSrc = NULL;
      foreach ( $part as $node ) {
         try {
            $part = new AggregatorPart( $node );
            $this->gParts[ $idx++ ] = $part;
         } catch ( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
         }
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      $documentDOM = new DomDocument();
      $root = $documentDOM->createElement( $this->gElement );
      $root = $documentDOM->appendChild( $root );
      
      $validAggregate = CacheFile::CACHE_VALID;
      $aggregateHash = "";
      
      
      for ( $i = 0; $i < count( $this->gParts ); $i++ ) {
         $part = $this->gParts[ $i ];
         $rawString = $part->getSrc();
         $aggregateHash = md5( $aggregateHash . $rawString );
         
         
         
         $stringExpansionCode = Environment::getSource( $rawString );
         switch ( $stringExpansionCode ) {
            case Environment::FILE_READ :
               $expandedString = StringResolver::removePseudoProtocols( $rawString );
               $expandedString = StringResolver::expandString( $inVariableStack, $expandedString );
               $scrapDOM = new DomDocument();
               if ( !$scrapDOM->load( $expandedString ) ) {
                  throw new RunException( "Could not load scrap from file '$expandedString'" );
               }
               break;
            case Environment::SUBMIT_ROOT :
               $expandedString = StringResolver::removePseudoProtocols( $rawString );
               $expandedString = StringResolver::expandString( $inVariableStack, $expandedString );
               
               $outputStream = new OutputStream( OutputStream::BUFFER );
               
               $sitemap = Environment::$sitemapStack->root();
               $oldOutputStream = $sitemap->getOutputStream();
               $sitemap->setOutputStream( $outputStream );
               
               $sitemap->run( $expandedString, $inQueryString, true );
               
               
               $scrapDOM = new DomDocument();
               if ( $scrapDOM->loadXML( $outputStream->getBuffer() ) === false ) {
                  throw new UserException( "Cannot load XML stream", PalooseException::SITEMAP_AGGREGATE_PARSE_ERROR );
               }
               
               $sitemap->setOutputStream( $oldOutputStream );
               break;
            case Environment::SUBMIT_CURRENT :
               $expandedString = StringResolver::removePseudoProtocols( $rawString );
               $expandedString = StringResolver::expandString( $inVariableStack, $expandedString );
               
               $outputStream = new OutputStream( OutputStream::BUFFER );
               
               $sitemap = Environment::$sitemapStack->peek();
               $oldOutputStream = $sitemap->getOutputStream();
               $sitemap->setOutputStream( $outputStream );
               
               $sitemap->run( $expandedString, $inQueryString, true );
               
               $scrapDOM = new DomDocument();
               if ( $scrapDOM->loadXML( $outputStream->getBuffer() ) === false ) {
                  throw new UserException( "Cannot load XML stream (no root tag in part?)", PalooseException::SITEMAP_AGGREGATE_RUN_ERROR );
               }
               
               $sitemap->setOutputStream( $oldOutputStream );
               break;
            default :
               throw new RunException( "Unknown return code '$stringExpansionCode' for string expansion source processing '$rawString'", PalooseException::UNKNOWN_STRING_EXPANSION_TYPE );
         }
         
         if ( $scrapDOM->documentElement->getAttribute( "__status" ) == CacheFile::CACHE_INVALID ) {
            $validAggregate = CacheFile::CACHE_INVALID;
         }
         $xpath = new domxpath( $scrapDOM );
         if ( $part->getStripRoot() ) {
             
            $nodeList = $xpath->query( "/*/*" );
         } else {
             
            $nodeList = $xpath->query( "/*" );
         }
         $dom = new DomDocument();
         
         
         $elementNode = $dom->createElement( $part->getElement() );
         $dom->appendChild( $elementNode );
         
         foreach ( $nodeList as $node ) {
            $newNode = $dom->importNode( $node, true );
            $elementNode->appendChild( $newNode );
         }
         
         try {
             $xpath = new domxpath( $dom );
            $nodeList = $xpath->query( "/*" );
            foreach ( $nodeList as $node ) {
               $newNode = $documentDOM->importNode( $node, true );
               $root->appendChild( $newNode );
            }
         } catch ( DOMException $e ) {
            throw new RunException( $e->getMessage(), $e->getCode );
         }
       
      } 
            
      $documentDOM->documentElement->setAttribute( "__status", $validAggregate );
      $documentDOM->documentElement->setAttribute( "__file", $aggregateHash );
      
      return $documentDOM;
   }
   
   
   
   public function toString()
   {
      if ( $this->gParts != NULL ) {
         $mess = "    <aggregate element='{$this->gElement}'>\n";
         foreach ( $this->gParts as $node ) {
            $mess .= $node->toString();
         }
         $mess .= "    </aggregate>\n";
      }
      return $mess;
   }
}
?>
