<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
 
class DirectoryTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
   
   private $gDepth = 1;
   
   private $gStartDir = 1;
   
   private $gDateFormat = ""; 
   
   
   private $gReverse = false;
   
   private $gInclude = "/.*/";
   
   private $gExclude = "";
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
       try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $documentDOM = $inDOM;
      $rootNode = $documentDOM->documentElement;
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      try {
         $this->processNode( $inVariableStack, $rootNode, $newElement );
         $this->gTranslatedDOM->formatOutput = true;
         
         return $this->gTranslatedDOM;
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
   
   
   
   private function processNode( VariableStack $inVariableStack, DOMNode $inParentNode, DOMNode &$inNewParentNode )
   {
      
      $folderTagFound = false;
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         $encodedAttribs = array();
         
         if ( $inParentNode->hasAttributes() and $inParentNode->nodeName == "pcms:folder" ) {
            $folderTagFound = true;
            
         } else if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         }
      }
            
      
      if ( $folderTagFound === true ) {
	      $inParentNode = $this->processFolderData( $inParentNode );
      }
      
      
      $children = $inParentNode->childNodes;
      foreach ( $children as $child ) {
         
         if ( $child->nodeType == XML_ELEMENT_NODE ) {
            
            
            $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
            
            $this->processNode( $inVariableStack, $child, $newElementNode );
            $inNewParentNode->appendChild( $newElementNode );
         } else if ( $child->nodeType == XML_TEXT_NODE ) {
            
            
            $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
            $inNewParentNode->appendChild( $newTextNode );
         }
      }
   }
   
   
   
 
   private function processFolderData( DOMNode $inSourceNode )
   {
      
      $dom = new DOMDocument();
	  $rootElement = $dom->appendChild( $dom->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:listing' ) );
      
        if ( $inSourceNode->hasAttributes() ) {
            foreach ( $inSourceNode->attributes as $attribute ) {
               $attribName = $attribute->name;
               $attribValue = $attribute->value;
               $rootElement->setAttribute( $attribName, $attribValue );
            }
         }
               
         $this->gStartDir = $inSourceNode->getAttribute( "src" );
         
         if ( $inSourceNode->hasAttribute(  "depth"  ) ) {
            $this->gDepth = $inSourceNode->getAttribute( "depth" );
         } else {
		    $this->gDepth = 1;
         }
         if ( $inSourceNode->hasAttribute(  "dateFormat"  ) ) {
            $this->gDateFormat = $inSourceNode->getAttribute( "dateFormat" );
         }
         if ( $inSourceNode->hasAttribute(  "include"  ) ) {
            $this->gInclude = $inSourceNode->getAttribute( "include" );
         }
         if ( $inSourceNode->hasAttribute(  "exclude"  ) ) {
            $this->gExclude = $inSourceNode->getAttribute( "exclude" );
         }
         if ( $inSourceNode->hasAttribute(  "reverse"  ) ) {
            $this->gReverse = $inSourceNode->getAttribute( "reverse" );
         }
          
		$directoryDOM = $this->scanDirectory( $dom, $rootElement, $this->gStartDir, $this->gDepth );
      
     return $directoryDOM;
   }
   
   
   
   private function scanDirectory( $inDOMDocument, $inRootElement, $inStartDirectory, $inDepth )
   {
      
	  $numberFound = 0;
      $inDepth--;
      $element = NULL;
      
      
      $dirFileListing = scandir( $inStartDirectory, ( $this->gReverse === false ) ? 0 : 1 );
      foreach( $dirFileListing as $key => $value ) {
         $fullName = "$inStartDirectory/$value";
         
         if ( $value == '.'
               || $value == '..'
               || ( strlen( $this->gInclude ) > 0 && preg_match( $this->gInclude, $value ) == 0 )
               || ( strlen( $this->gExclude ) > 0 && preg_match( $this->gExclude, $value ) != 0 ) ) {
            
         } else if ( is_dir( $fullName ) ) {
            $element = $inRootElement->appendChild( $inDOMDocument->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:directory' ) );
            $element->setAttribute( "name" , $fullName );
            $element->setAttribute( "lastmodified" , $this->getLastModified( $fullName ) );
            $element->setAttribute( "size" , filesize( $fullName ) );
            if ( $inDepth > 0 ) {
               $scanString = $this->scanDirectory( $inDOMDocument, $element, $fullName, $inDepth );
            } else {
               $scanString = "";
            }
         } else {
            $numberFound++;
            $element = $inRootElement->appendChild( $inDOMDocument->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:file' ) );
            $element->setAttribute( "name" , $value );
            $element->setAttribute( "lastmodified" , $this->getLastModified( $fullName ) );
            $element->setAttribute( "size" , filesize( $fullName ) );
         }
      }
      
      if ( $numberFound == 0 ) {
		   $element = $inParentElement->appendChild( $inDOMDocument->createElementNS( Environment::$configuration[ 'dirNamespace' ], 'dir:empty' ) );
	   }
      return $inDOMDocument;
   }
   
   
   
   private function getLastModified( $inDirectory )
   {
      return date( $this->gDateFormat, filemtime( $inDirectory ));
   }
   
   
   
   
  
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
}
 
class _DirectoryTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "DirectoryTransformer";
   }
}
?>
