<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
class SourceWritingTransformer extends TransformerPipeElement implements PipeElementInterface
{
   
   
   
   const WRITE_SOURCE_ACTION_NEW = 1;
   const WRITE_SOURCE_ACTION_OVERWRITTEN = 2;
   const WRITE_SOURCE_ACTION_NONE = 3;
   
   const WRITE_SOURCE_BEHAVIOUR_WRITE = 1;
   const WRITE_SOURCE_BEHAVIOUR_INSERT = 2;
   const WRITE_SOURCE_BEHAVIOUR_DELETE = 3;
   
   const WRITE_SOURCE_EXECUTION_SUCCESS = 1;
   const WRITE_SOURCE_EXECUTION_FAILURE = 2;
   
   const WRITE_SOURCE_SERIALIZER_XML = 1;
   
   
   private $gCreateFlag = false;
   
   
   private $gOverwriteFlag = true;
   
   const IMPORT_SUBTREE = true;
   
   const INITIAL = 0;
   const APPENDING_NODE = 1;
   const REPLACING_NODE = 2;
   const DELETING_NODE = 3;
   const NO_ACTION = 4;
   
   
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      
      $documentDOM = $inDOM;
      $rootNode = $documentDOM->documentElement;
      
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      
      $this->gTranslatedDOM->appendChild( $newElement );
      try {
         $this->processNode( $inVariableStack, null, $rootNode, $newElement );
         return $this->gTranslatedDOM;
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
      
   }
   
   
   
   
   
   private function processNode( VariableStack $inVariableStack, $inCatalogueDOM, DOMNode $inParentNode, DOMNode &$inNewParentNode )
   {
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
               
            }
         }
      }
      
      
      
      $children = $inParentNode->childNodes;
      
      
      try {
         foreach ( $children as $child ) {
            if ( $child->nodeName == "source:write" ) {
               $replacementDOM = $this->processWrite( $child );
               
               $replacementRoot = $replacementDOM->documentElement;
               
               $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
               $inNewParentNode->appendChild( $newElementNode );
            } else if ( $child->nodeName == "source:insert" ) {
               
               if ( $child->hasAttributes() ) {
                  foreach ( $child->attributes as $attribute ) {
                     $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
                     $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
                     $inNewParentNode->setAttribute( $attribName, $attribValue );
                  }
               }
               
               $replacementDOM = $this->processInsert( $child );
               
               $replacementRoot = $replacementDOM->documentElement;
               
               $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
               $inNewParentNode->appendChild( $newElementNode );
            } else if ( $child->nodeName == "source:delete" ) {
               $replacementDOM = $this->processDelete( $child );
               
               $replacementRoot = $replacementDOM->documentElement;
               
               $newElementNode = $this->gTranslatedDOM->importNode( $replacementRoot, true );
               $inNewParentNode->appendChild( $newElementNode );
            } else {               
               if ( $child->nodeType == XML_ELEMENT_NODE ) {
                  
                  $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
                  
                  $this->processNode( $inVariableStack, $inCatalogueDOM, $child, $newElementNode );
                  $inNewParentNode->appendChild( $newElementNode );
               } else if ( $child->nodeType == XML_TEXT_NODE ) {
                  
                  $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
                  $inNewParentNode->appendChild( $newTextNode );
               }
            }
         }
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
   
   
   
   
   
   private function processWrite( DOMNode $inSourceNode )
   {
      
      $sourceDom = new DOMDocument();
      $sourceDom->appendChild( $sourceDom->importNode( $inSourceNode, 1 ) );
      
      $xpath = new DOMXpath( $sourceDom );
      $xpath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      $filename = Utilities::getXPathListStringItem( 0, $xpath, "//source:source" );
      $filename = StringResolver::expandPseudoProtocols( $filename );
      $childXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:path" );
      
      $this->gCreateFlag = Utilities::normalizeBoolean( Utilities::getXPathListStringItem( 0, $xpath, "//@create" ) );
      
      
      if ( !file_exists( $filename ) and !$this->gCreateFlag ) {
         throw new RunException( "No file to receive source write fragment (try create attrib?)",
                                PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
      }
      
      
      $fragmentDOM = new DOMDocument();
      $fragmentDOM->appendChild( $fragmentDOM->importNode( Utilities::getXPathListNode( 0, $xpath, "//source:fragment/*" ), 1 ) );
      
      $documentDOM = new DOMDocument();
      try {
         if ( strlen( $childXpath ) > 0 ) {
            $pathXML = $this->xpathToXML( $childXpath );
            $documentDOM->loadXML( $pathXML );
            $sourceXpath = new DOMXpath( $documentDOM );
            
            
            $parentNode = Utilities::getXPathListNode( 0, $sourceXpath, $childXpath );
            if ( $parentNode == NULL ) {
            }
            
            
            $documentDOM->importNode( $fragmentDOM->documentElement, true );
            $parentNode->appendChild( $documentDOM->importNode( $fragmentDOM->documentElement, true ) );
            
            
            if ( fopen( $filename, "wb" ) === false ) {
               throw new RunException( "Cannot write source write fragment (directory permissions?)",
                                      PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
            }
            
         } else {
            
            $nodeList = $xpath->query( "//source:fragment/*" );
            if ( $nodeList->length == 1 ) {
               $documentDOM->loadXML( $fragmentDOM->saveXML() );
            } else {
               throw new RunException( "Fragment must have one root element if no path declared",
                                      PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
            }
         }
         
         
         if ( !$documentDOM->save( $filename ) ) {
            $errorMessage = "Could not save document to file '$filename': $php_errormsg";
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                     self::WRITE_SOURCE_BEHAVIOUR_WRITE,
                                                     self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                     self::WRITE_SOURCE_SERIALIZER_XML,
                                                     $filename,
                                                     $errorMessage );
         } else {
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                     self::WRITE_SOURCE_BEHAVIOUR_WRITE,
                                                     self::WRITE_SOURCE_EXECUTION_SUCCESS,
                                                     self::WRITE_SOURCE_SERIALIZER_XML,
                                                     $filename,
                                                     "File written ok" );
         }
         
      } catch ( Exception $e ) {
         $errorMessage = "Problem processing source document in write-source: $e->getMessage()";
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                  self::WRITE_SOURCE_BEHAVIOUR_WRITE,
                                                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                  self::WRITE_SOURCE_SERIALIZER_XML,
                                                  $filename,
                                                  $errorMessage );
      }
      return $resultDOM;
   }
   
   
   
   
   
   private function processInsert( DOMNode $inSourceNode )
   {
      
      $sourceDom = new DOMDocument();
      $sourceDom->appendChild( $sourceDom->importNode( $inSourceNode, 1 ) );
      
      
      $sourceXpath = new DOMXpath( $sourceDom );
      $sourceXpath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      
      $filename = Utilities::getXPathListStringItem( 0, $sourceXpath, "//source:source" );
      $filename = StringResolver::expandPseudoProtocols( $filename );
      $emptyFilename = ( strlen( $filename ) == 0 );
      
      $pathXpath = Utilities::getXPathListStringItem( 0, $sourceXpath, "//source:path" );
      $emptyPath = ( strlen( $pathXpath ) == 0 );
      
      $replaceXpath = Utilities::getXPathListStringItem( 0, $sourceXpath, "//source:replace" );
      $emptyReplace = ( strlen( $replaceXpath ) == 0 );
      
      $reinsertXpath = Utilities::getXPathListStringItem( 0, $sourceXpath, "//source:reinsert" );
      $emptyReinsert = ( strlen( $reinsertXpath ) == 0 );
      
      $this->gCreateFlag = Utilities::normalizeBoolean( Utilities::getXPathListStringItem( 0, $sourceXpath, "/source:insert/@create" ) );
      $this->gOverwriteFlag = Utilities::normalizeBoolean( Utilities::getXPathListStringItem( 0, $sourceXpath, "/source:insert/@overwrite" ) );
      
      $fragmentNode = Utilities::getXPathListNode( 0, $sourceXpath, "//source:fragment/*" );
      $emptyFragment = ( $fragmentNode == NULL );
      
      $fragmentDOM = new DOMDocument();
      if ( !$emptyFragment ) {
         $fragmentDOM->importNode( Utilities::getXPathListNode( 0, $sourceXpath, "//source:fragment/*" ), 1 );
      }
      
      if ( !$emptyFragment )  {
      } else {
      }
      
      
      
      if ( $emptyFilename ) {
         $errorMessage = "Must have <source:source> defined in <source:insert>";
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                  self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                                                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                  self::WRITE_SOURCE_SERIALIZER_XML,
                                                  $filename,
                                                  $errorMessage );
         return $resultDOM;
      }
      
      if ( $emptyPath ) {
         $errorMessage = "Must have <source:path> defined in <source:insert>";
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                  self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                                                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                  self::WRITE_SOURCE_SERIALIZER_XML,
                                                  $filename,
                                                  $errorMessage );
         return $resultDOM;
      }
      
      
      if ( !file_exists( $filename ) and !$this->gCreateFlag ) {
         throw new RunException( "No file to receive source write fragment (try @create attrib in source:insert?)",
                                PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
      }
      
      
      
      
      
      
      $case = $this::INITIAL;
      if ( $emptyReplace && !$emptyFragment ) {
         
         $case = $this::APPENDING_NODE;
      } else if ( !$emptyReplace && !$emptyFragment && $this->gOverwriteFlag ) {
         
         
         $case = $this::REPLACING_NODE;
      } else if ( !$emptyReplace && $emptyFragment && $this->gOverwriteFlag ) {
         
         $case = $this::DELETING_NODE;
      } else if ( !$emptyReplace && !$this->gOverwriteFlag ) {
         
         $case = $this::NO_ACTION;
      } else if ( !$emptyReplace && !$emptyFragment ) {
         
         
         $case = $this::APPENDING_NODE;
      } else if ( $emptyReinsert ) {
         
         $case = $this::NO_ACTION;
      }
      
      if ( !file_exists( $filename ) ) {
         throw new RunException( "File '$filename' does not exist", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
      }
      
      
      $documentDOM = new DOMDocument();
      if ( !$documentDOM->load( $filename ) ) {
         throw new RunException( "Could not load source document from file '$filename'", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
      }
      
      
      $documentXPath = new DOMXpath( $documentDOM );
      $documentXPath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      $writeFileFlag = false;
      
      
      try {
         
         switch ( $case ) {
            case $this::APPENDING_NODE :
               
               
               $fragmentNode = Utilities::getXPathListNode( 0, $sourceXpath, "//source:fragment/*" );
               if ( $fragmentNode == NULL ) {
                  throw new Exception( "Cannot find fragment node in source" );
               }
               $fragmentDOM->appendChild( $fragmentDOM->importNode( $fragmentNode, self::IMPORT_SUBTREE ) );
               
               
               $parentNode = Utilities::getXPathListNode( 0, $documentXPath, $pathXpath );
               if ( $parentNode == NULL ) {
                  throw new Exception( "Cannot find parent of node to append as child in '$pathXpath'" );
               }
               
               
               $parentNode->appendChild( $documentDOM->importNode( $fragmentDOM->documentElement, self::IMPORT_SUBTREE ) );
               $writeFileFlag = true;
               break;
               
            case $this::REPLACING_NODE:
               
               $nodeToReplace = Utilities::getXPathListNode( 0, $documentXPath, $replaceXpath );
               if ( $nodeToReplace == NULL ) {
                  throw new Exception( "Cannot find node to replace: $replaceXpath" );
               }
               
               
               $parentNode = Utilities::getXPathListNode( 0, $documentXPath, $pathXpath );
               if ( $parentNode == NULL ) {
                  throw new Exception( "Cannot find parent of node to replace in '$pathXpath'" );
               }
               
               $fragmentNode = Utilities::getXPathListNode( 0, $sourceXpath, "//source:fragment/*" );
               if ( $fragmentNode != NULL ) {
                  $fragmentDOM->appendChild( $fragmentDOM->importNode( $fragmentNode, self::IMPORT_SUBTREE ) );
               }
               
               if ( !$parentNode->removeChild( $nodeToReplace ) ) {
                  throw new RunException( "Could not remove child: " . $nodeToReplace->getNodePath(), PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
               }
               
               if ( !$emptyFragment ) {
                  $parentNode->appendChild( $documentDOM->importNode( $fragmentDOM->documentElement, self::IMPORT_SUBTREE ) );
               }
               $writeFileFlag = true;
               break;
               
            case $this::DELETING_NODE:
               
               $nodeToDelete = Utilities::getXPathListNode( 0, $documentXPath, $replaceXpath );
               if ( $nodeToDelete == NULL ) {
                  throw new Exception( "Cannot find node to delete: $pathXpath" );
               }
               
               
               $parentNode = Utilities::getXPathListNode( 0, $documentXPath, $pathXpath );
               if ( $parentNode == NULL ) {
                  throw new Exception( "Cannot find parent of node to delete in '$pathXpath'" );
               }
               
               if ( !$parentNode->removeChild( $nodeToDelete ) ) {
                  throw new RunException( "Could not remove child: " . $nodeToReplace->getNodePath(), PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
               }
               
               $writeFileFlag = true;
               break;
               
            case $this::NO_ACTION:
               
               $writeFileFlag = false;
               break;
               
         } 
         
         
         
         
         if ( $writeFileFlag ) {
            
            if ( !$documentDOM->save( $filename ) ) {
                 $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                        self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                                                        self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                        self::WRITE_SOURCE_SERIALIZER_XML,
                                                        $filename,
                                                        "Could not save document to file '$filename': " . $php_errormsg );
            } else {
               $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                        self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                                                        self::WRITE_SOURCE_EXECUTION_SUCCESS,
                                                        self::WRITE_SOURCE_SERIALIZER_XML,
                                                        $filename,
                                                        "File written ok" );
            }
         } else {
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                     self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                                                     self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                     self::WRITE_SOURCE_SERIALIZER_XML,
                                                     $filename,
                                                     "Did not write to '$filename': " );
         }
      } catch ( Exception $e ) {
         $errorMessage = "Problem processing source document in write-source: $e->getMessage() ";
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                  self::WRITE_SOURCE_BEHAVIOUR_INSERT,
                                                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                  self::WRITE_SOURCE_SERIALIZER_XML,
                                                  $filename,
                                                  $errorMessage );
      }
      return $resultDOM;
   }
   
   
   
   
   
   private function processDelete( DOMNode $inSourceNode )
   {
      
      $sourceDom = new DOMDocument();
      $sourceDom->appendChild( $sourceDom->importNode( $inSourceNode, 1 ) );
      
      
      
      $xpath = new DOMXpath( $sourceDom );
      $xpath->registerNamespace( "source", Environment::$configuration[ 'sourceNamespace' ] );
      
      $filename = StringResolver::expandPseudoProtocols( Utilities::getXPathListStringItem( 0, $xpath, "//source:source" ) );
      $childXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:path" );
      $replaceXpath = Utilities::getXPathListStringItem( 0, $xpath, "//source:replace" );
      
      try {
         if ( !file_exists( $filename ) ) {
            $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                     self::WRITE_SOURCE_BEHAVIOUR_DELETE,
                                                     self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                     self::WRITE_SOURCE_SERIALIZER_XML,
                                                     $filename,
                                                     "File does not exist" );
         } else {
            
            if ( !unlink( $filename ) ) {
               $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                        self::WRITE_SOURCE_BEHAVIOUR_DELETE,
                                                        self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                        self::WRITE_SOURCE_SERIALIZER_XML,
                                                        $filename,
                                                        "Could not delete file '$filename': " . $php_errormsg );
            } else {
               $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                        self::WRITE_SOURCE_BEHAVIOUR_DELETE,
                                                        self::WRITE_SOURCE_EXECUTION_SUCCESS,
                                                        self::WRITE_SOURCE_SERIALIZER_XML,
                                                        $filename,
                                                        "File delete ok" );
            }
         }
      } catch ( Exception $e ) {
         $errorMessage = "Problem processing source document in write-source: $e->getMessage() ";
         $resultDOM = $this->formatResultFragment( self::WRITE_SOURCE_ACTION_NEW,
                                                  self::WRITE_SOURCE_BEHAVIOUR_DELETE,
                                                  self::WRITE_SOURCE_EXECUTION_FAILURE,
                                                  self::WRITE_SOURCE_SERIALIZER_XML,
                                                  $filename,
                                                  $errorMessage );
      }
      return $resultDOM;
   }
   
   
   
   
   private function formatResultFragment( $inAction, $inBehaviour, $inExecution, $inSerializer, $inSource, $inMessage )
   {
      $ns = Environment::$configuration[ 'sourceNamespace' ];
      
      $dom = new DOMDocument();
      $root = $dom->createElementNS( $ns, 'sourceResult' );
      $dom->appendChild( $root );
      switch ( $inAction ) {
         case self::WRITE_SOURCE_ACTION_NEW : $action = 'new'; break;
         case self::WRITE_SOURCE_ACTION_OVERWRITTEN : $action = 'overwritten'; break;
         case self::WRITE_SOURCE_ACTION_NONE : $action = 'none'; break;
         default: $action = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'action', $action ) );
      
      switch ( $inBehaviour ) {
         case self::WRITE_SOURCE_BEHAVIOUR_WRITE : $behaviour = 'write'; break;
         case self::WRITE_SOURCE_BEHAVIOUR_INSERT : $behaviour = 'insert'; break;
         default: $behaviour = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'behaviour', $behaviour ) );
      
      switch ( $inExecution ) {
         case self::WRITE_SOURCE_EXECUTION_SUCCESS : $execution = 'success'; break;
         case self::WRITE_SOURCE_EXECUTION_FAILURE : $execution = 'failure'; break;
         default: $execution = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'execution', $execution ) );
      
      switch ( $inSerializer ) {
         case self::WRITE_SOURCE_SERIALIZER_XML : $serializer = 'xml'; break;
         default: $serializer = "";
      }
      $root->appendChild( $dom->createElementNS( $ns, 'serializer', $serializer ) );
      $root->appendChild( $dom->createElementNS( $ns, 'source', $inSource ) );
      $root->appendChild( $dom->createElementNS( $ns, 'message', $inMessage ) );
      return $dom;
   }
   
   
   
   
   
   private function xpathToXML( $inXpathExpr )
   {
      $firstTag = "";
      $remainder = "";
      preg_match( "%/([^/]+)(.*)%", $inXpathExpr, $matches );
      if ( isset( $matches[ 1 ] ) ) { $firstTag = $matches[ 1 ]; }
      if ( isset( $matches[ 2 ] ) ) { $remainder = $matches[ 2 ]; }
      if ( strlen( $remainder ) > 0 ) {
         return "<$firstTag>" . $this->xpathToXML( $remainder ) . "</$firstTag>";
      } else {
         return "<$firstTag/>";
      }
   }
   
   
   
   
   
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
   
}
class _SourceWritingTransformer extends Transformer {
   
   
   
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "SourceWritingTransformer";
   }
   
}
?>
